@@ -114,6 +114,101 @@ static int zopen(char *file, int mode, int *size)
 }
 #endif /* VMS */
 
+#if MOST_HAS_MMAP
+#ifdef HAVE_SYS_MMAN_H
+# include <sys/mman.h>
+#endif
+
+#ifndef MAP_FAILED
+# define MAP_FAILED	-1
+#endif
+static int try_mmap_buffer (int fd)
+{
+   unsigned char *addr;
+   struct stat st;
+
+   Most_Buf->is_mmaped = 0;
+   
+   if (Most_Disable_MMap)
+     return -1;
+
+   if (-1 == fstat (fd, &st))
+     return -1;
+   
+   addr = (unsigned char *)mmap (NULL,st.st_size,PROT_READ,MAP_SHARED,fd,0);
+   if (addr == (unsigned char *)MAP_FAILED)
+     return -1;
+   
+   Most_Buf->fd = fd;
+   Most_Eob = Most_Beg = Most_Buf->end = Most_Buf->beg = addr;
+   Most_Buf->size = 0;
+   Most_Buf->mmap_size = st.st_size;
+   Most_Buf->is_mmaped = 1;
+
+   return 0;
+}
+
+static int unmmap_buffer (Most_Buffer_Type *b)
+{
+   if (b->is_mmaped == 0)
+     return 0;
+   
+   if (b->beg != NULL)
+     munmap ((char *)b->beg, b->size);
+   
+   b->end = b->beg = NULL;
+   if (b == Most_Buf)
+     {
+	Most_Beg = Most_Eob = NULL;
+	Most_C_Line = 0;
+	Most_Num_Lines = Most_C_Line = 0;
+     }
+   return 0;
+}
+
+static int resync_mmap (void)
+{   
+   struct stat st;
+
+   if (Most_Buf->fd == -1)
+     return 0;
+   if (-1 == fstat (Most_Buf->fd, &st))
+     return -1;
+   
+   if ((unsigned int) st.st_size == Most_Buf->mmap_size)
+     return 0;
+
+   (void) unmmap_buffer (Most_Buf);
+   if (-1 == try_mmap_buffer (Most_Buf->fd))
+     return -1;
+   
+   return 0;
+}
+
+
+#endif				       /* MOST_HAS_MMAP */
+
+int most_close_buffer_file (Most_Buffer_Type *b)
+{
+   if (b->fd != -1)
+     {
+#if !defined(VMS)
+	if (b->fp != NULL) pclose (b->fp); 
+	else
+#endif
+	  close(b->fd);
+     }
+#if MOST_HAS_MMAP
+   unmmap_buffer (b);
+#endif
+   if (b->beg != NULL) 
+     SLFREE(b->beg);
+   b->beg = NULL;
+   b->fd = -1;
+   
+   return 0;
+}
+
 /* If file[0] == 0, the file represents stdin */
 static int insert_file(char *file)
 {
@@ -144,6 +239,10 @@ static int insert_file(char *file)
           /* if (fd < 0) fd = open(file,O_RDONLY); */
 #else
 	fd = zopen(file, O_RDONLY, &size);
+# if MOST_HAS_MMAP
+	if (0 == try_mmap_buffer (fd))
+	  return most_read_file_dsc (1);
+# endif
 #endif
      }
 
@@ -176,7 +275,80 @@ static void update_buffer_windows (Most_Buffer_Type *b)
    while (w != Most_Win);
 }
 
+static void examine_file_contents (void)
+{
+   unsigned char *pos, *pos_max;
+
+   if (Most_B_Opt || Most_K_Opt)
+     return;
+   
+   pos = Most_Beg;
+   pos_max = pos + 32;
+   if (pos_max > Most_Eob)
+     pos_max = Most_Eob;
+	
+   while (pos < pos_max)
+     {
+	if (0 == (*pos & 0x7F)) Most_B_Opt = 1;
+	pos++;
+     }
+	
+   /* Check for a FITS file.  The header looks like ASCII. */
+   if (Most_B_Opt == 0)
+     {
+	pos = Most_Beg;
+	pos_max = Most_Eob;
+	if ((pos_max > pos + 80)
+	    && (0 == strncmp ((char *) pos, "SIMPLE  =", 9)))
+	  {
+	     Most_V_Opt = 1;
+	     Most_W_Opt = 1;
+	  }
+     }
+}
+
 static int First_Time_Hack = 0;	       /* true if reading file for first time */
+
+#if MOST_HAS_MMAP
+static int read_mmap_file_dsc (int many)
+{
+   unsigned int size;
+
+   size = Most_Buf->size;
+
+   if (many == -1)
+     {
+	if (-1 == resync_mmap ())
+	  return -1;
+     }
+       
+   if (size == Most_Buf->mmap_size)
+     return 0;
+   
+   if (many < 0)
+     size = Most_Buf->mmap_size;
+   else
+     {
+	size += many * 0xFFFF;
+	if (size > Most_Buf->mmap_size)
+	  size = Most_Buf->mmap_size;
+     }
+   
+   Most_Eob = Most_Buf->end = Most_Buf->beg + size;
+   Most_Buf->size = size;
+
+   most_flush_message ("Mapping file...");
+
+   if (First_Time_Hack)
+     examine_file_contents ();
+   
+   Most_Num_Lines = most_count_lines (Most_Beg, Most_Eob);
+   most_message(Most_Global_Msg, 0);
+   return 1;
+}
+
+#endif
+
 /* if read something, return non zero (1) */
 int most_read_file_dsc (int many)
 {
@@ -189,6 +361,19 @@ int most_read_file_dsc (int many)
 
    if (fd < 0) return 0;
 
+#if MOST_HAS_MMAP
+   if (Most_Buf->is_mmaped)
+     {
+	int ret = read_mmap_file_dsc (many);
+	if (-1 != ret)
+	  return ret;
+	
+	if (Most_Buf->is_mmaped)
+	  return -1;
+	/* No longer mapped..... */
+     }
+#endif
+
    dsize = 0x3FFF;
 
    while (many--)
@@ -226,10 +411,23 @@ int most_read_file_dsc (int many)
 #endif
 		  break;
 	       }
-
+	     
+	     if (i == 0)
+	       {
+		  /* EOF */
+		  /* Under linux, for some reason bash when invoked by
+		   * man will reset the terminal.  Apparantly this is a
+		   * result of most being orphaned.
+		   */
+#ifdef __unix__
+		  if (fd == 0)
+		    (void) most_reinit_terminal ();
+#endif
+		  break;
+	       }
 	     n += i;
 	     pos += i;
-	     if ((n >= dsize) || (i == 0))
+	     if (n >= dsize)
 	       break;
 	  }
 
@@ -245,33 +443,8 @@ int most_read_file_dsc (int many)
 	  }
      }
 
-   if (First_Time_Hack 
-       && (Most_B_Opt == 0) && (Most_K_Opt == 0))
-     {
-	unsigned char *pos_max;
-
-	pos = Most_Beg;
-	pos_max = pos + 32;
-	if (pos_max > Most_Eob)
-	  pos_max = Most_Eob;
-	
-	while (pos < pos_max)
-	  {
-	     if (0 == (*pos & 0x7F)) Most_B_Opt = 1;
-	     pos++;
-	  }
-	
-	/* Check for a FITS file.  The header looks like ASCII. */
-	if (Most_B_Opt == 0)
-	  {
-	     pos = Most_Beg;
-	     if (0 == strncmp ((char *) pos, "SIMPLE  =", 9))
-	       {
-		  Most_V_Opt = 1;
-		  Most_W_Opt = 1;
-	       }
-	  }
-     }
+   if (First_Time_Hack)
+     examine_file_contents ();
 
    if (n || First_Time_Hack)
      Most_Num_Lines = most_count_lines (Most_Beg, Most_Eob);
