@@ -10,6 +10,10 @@ extern int Most_W_Opt;             /* Wrap mode */
 extern int Most_K_Opt;             /* Kanji mode */
 extern int Most_Z_Opt;             /* gunzip-on-the-fly */
 
+#if MOST_HAS_MMAP
+extern int Most_Disable_MMap;
+#endif
+
 extern int Most_Secure_Mode;
 extern int Most_Captive_Mode;
 extern char *Most_Program;
