@@ -234,7 +234,7 @@ static int get_scroll(int *line)
 	     Most_C_Line = 1;
 	     return 0;
 	  }
-	most_message("Top of Buffer.",1);
+	most_message("Top of Buffer.",0);
 	return(0);
      }
 
@@ -275,7 +275,7 @@ static int get_scroll(int *line)
 		  Most_C_Line = 1;
 		  return 0;
 	       }
-	     most_message("Top of buffer.",1);
+	     most_message("Top of buffer.",0);
 	  }
      }
    return(n);
@@ -506,13 +506,20 @@ static void update_status1 (void)
    unsigned int len;
    unsigned int num_chars;
    unsigned int field_width, info_len;
+   unsigned char *eob;
    int r, x;
 
-   if (Most_Eob == Most_Beg) x = 100;
+   eob = Most_Eob;
+#if MOST_HAS_MMAP
+   if (Most_Buf->is_mmaped)
+     eob = Most_Beg + Most_Buf->mmap_size;
+#endif
+   
+   if (eob == Most_Beg) x = 100;
    else
      {
 	x = Most_C_Offset * 100;
-	x = x / (Most_Eob - Most_Beg);
+	x = x / (eob - Most_Beg);
      }
 
    if (Most_C_Line + (Most_Win->bot - Most_Win->top + 1) >= Most_Num_Lines)
@@ -799,15 +806,8 @@ void most_free_window_buffer (void)
      }
    b = w->buf;
    if (b == NULL) return;
-   if ((b->fd != -1) /* && (b->fd != 0) */ )
-     {
-#if !defined(VMS)
-	if (b->fp != NULL) pclose (b->fp); 
-	else
-#endif
-	  close(b->fd);
-     }
-   if (b->beg != NULL) SLFREE(b->beg);
+   
+   most_close_buffer_file (b);
    Most_Win->buf = NULL;
    SLFREE(b);
 }
